-- fsm_lamp.vhd

library ieee;
use ieee.std_logic_1164.all;

entity fsm_lamp is
  port
  (
    clk : in std_logic;
    ta : in std_logic;
    led : out std_logic
  );
end fsm_lamp;

architecture impl of fsm_lamp is

type lamp_state_type is (aus, h1, ein, h2);
signal lamp_state_reg : lamp_state_type := aus;
signal lamp_state_next : lamp_state_type;

signal led_reg : std_logic := '0';
signal led_next : std_logic;

begin
  upd_proc: process (clk)
  begin
    if rising_edge(clk) then
      lamp_state_reg <= lamp_state_next;
      led_reg <= led_next;
    end if;
  end process upd_proc;
  
  state_proc: process (lamp_state_reg, ta)
  begin
    lamp_state_next <= lamp_state_reg; -- default, no change
    led_next <= '0'; --default
    case lamp_state_reg is
      when aus =>
        led_next <= '0';
        if ta = '1' then
          lamp_state_next <= h1;
        end if;
      when h1 =>
        if ta = '0' then
          lamp_state_next <= ein;
        end if;
      when ein =>
        led_next <= '1';
        if ta = '1' then
          lamp_state_next <= h2;
        end if;
      when h2 =>
        led_next <= '1';
        if ta = '0' then
          lamp_state_next <= aus;
        end if;
    end case;
  end process state_proc;
  
  led <= led_reg;
  
end impl;
