-- fsm_ta.vhd

library ieee;
use ieee.std_logic_1164.all;

entity fsm_ta is
  port
  (
    clk : in std_logic;
    ta : in std_logic;
    led : out std_logic
  );
end fsm_ta;

architecture impl of fsm_ta is

type lamp_state_type is (aus, ein);
signal lamp_state_reg : lamp_state_type := aus;
signal lamp_state_next : lamp_state_type;

type ta_state_type is (released, pressed, pulse);
signal ta_state_reg : ta_state_type := released;
signal ta_state_next : ta_state_type;
signal ta_pulse : std_logic := '0';

signal led_reg : std_logic := '0';
signal led_next : std_logic;

begin
  upd_proc: process (clk)
  begin
    if rising_edge(clk) then
      lamp_state_reg <= lamp_state_next;
      ta_state_reg <= ta_state_next;
      led_reg <= led_next;
    end if;
  end process upd_proc;
  
  state_proc: process (lamp_state_reg, ta_pulse)
  begin
    lamp_state_next <= lamp_state_reg; -- default, no change
    case lamp_state_reg is
      when aus =>
        led_next <= '0';
        if ta_pulse = '1' then
          lamp_state_next <= ein;
        end if;
      when ein =>
        led_next <= '1';
        if ta_pulse = '1' then
          lamp_state_next <= aus;
        end if;
    end case;
  end process state_proc;

  ta_proc : process (ta_state_reg, ta)
  begin
    ta_state_next <= ta_state_reg; -- default
    ta_pulse <= '0'; --default
    case ta_state_reg is
      when released =>
        if ta = '1' then
          ta_state_next <= pressed;
        end if;
      when pressed =>
        if ta = '0' then
          ta_state_next <= pulse;
        end if;
      when pulse =>
        ta_pulse <= '1';
        ta_state_next <= released;
    end case;
  end process;

  led <= led_reg;  
end impl;