-- sseg_basys3_test2.vhd

library ieee;
use ieee.std_logic_1164.all;


entity sseg_basys3_test2 is
  port
  (
    clk : in std_logic;
    sw : in std_logic_vector(11 downto 0);
    seg : out std_logic_vector(6 downto 0);
    dp : out std_logic;
    an : out std_logic_vector(3 downto 0)
  );
end sseg_basys3_test2;

architecture impl of sseg_basys3_test2 is
component sseg_control is
  port
  (
    clk : in std_logic;
    disp_data : in std_logic_vector(31 downto 0);
    seg_data : out std_logic_vector(7 downto 0);
    digit_sel : out std_logic_vector(3 downto 0)
  );
end component;

component bin6tobcd is
  port
  (
    bin6 : in std_logic_vector(5 downto 0);
    ones : out std_logic_vector(3 downto 0);
    tens : out std_logic_vector(2 downto 0)
  );
end component;

component hex2sseg is
  port
  (
    hexnum : in std_logic_vector(3 downto 0);
    decpoint : in std_logic;
    sevseg : out std_logic_vector(7 downto 0) -- dp : bit 7; g: bit 6 ; ... a : bit 0
  );
end component;

signal disp_buf : std_logic_vector(31 downto 0);
signal seg_buf : std_logic_vector(7 downto 0);
signal digit_sel_buf : std_logic_vector(3 downto 0);

signal digit0_data : std_logic_vector(3 downto 0);
signal digit1_data : std_logic_vector(3 downto 0);
signal digit2_data : std_logic_vector(3 downto 0);
signal digit3_data : std_logic_vector(3 downto 0);

begin
  sseg_cntrl : sseg_control
  port map
  (
    clk => clk,
    disp_data => disp_buf,
    seg_data => seg_buf,
    digit_sel => digit_sel_buf
  );
  
  bintobcd0: bin6tobcd
  port map
  (
    bin6 => sw(5 downto 0),
    ones => digit0_data,
    tens => digit1_data(2 downto 0)
  );
  
  bintobcd1: bin6tobcd
  port map
  (
    bin6 => sw(11 downto 6),
    ones => digit2_data,
    tens => digit3_data(2 downto 0)
  );
  
  hex2sseg_00 : hex2sseg
  port map
  (
    hexnum => digit0_data,
    decpoint => '0',
    sevseg => disp_buf(7 downto 0)
  );
  
  hex2sseg_01 : hex2sseg
  port map
  (
    hexnum => digit1_data,
    decpoint => '0',
    sevseg => disp_buf(15 downto 8)
  );
  
  hex2sseg_02 : hex2sseg
  port map
  (
    hexnum => digit2_data,
    decpoint => '0',
    sevseg => disp_buf(23 downto 16)
  );
  
  hex2sseg_03 : hex2sseg
  port map
  (
    hexnum => digit3_data,
    decpoint => '0',
    sevseg => disp_buf(31 downto 24)
  );
   
  digit1_data(3) <= '0';
  digit3_data(3) <= '0';
  
  seg <= not seg_buf(6 downto 0);
  dp <= not seg_buf(7);
  an <= not digit_sel_buf;

end impl;