-- led_rot.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity led_rot is
  generic
  (
    led_cnt : integer := 16 -- number of LEDs
  );
  port
  (
    clk : in std_logic;
    led : out std_logic_vector(led_cnt-1 downto 0)
  );
end led_rot;

architecture impl of led_rot is

signal cntr_reg : unsigned(24 downto 0) := (others => '0');
signal cntr_next : unsigned(24 downto 0);
signal pattern_reg : std_logic_vector(led_cnt-1 downto 0) := (0|2|4|6 => '1', others => '0');
signal pattern_next : std_logic_vector(led_cnt-1 downto 0);

signal rot_pulse : std_logic;

begin
  process (clk)
  begin
    if rising_edge(clk) then
      cntr_reg <= cntr_next;
      if rot_pulse = '1' then
        pattern_reg <= pattern_next;
      end if;
    end if;
  end process;  

  cntr_next <= cntr_reg + 1;
  rot_pulse <= '1' when (cntr_reg = 0) else '0';
  pattern_next <= pattern_reg(led_cnt-2 downto 0) & pattern_reg(led_cnt-1);
  led <= pattern_reg;
end impl;
