-- dg_de0.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity dg_de0 is
  port
(
  clk50 : in std_logic; -- 50MHz clock
  btn0_n : in std_logic; -- reset button
  btn1_n : in std_logic; -- minute set button
  btn2_n : in std_logic; -- hour set button
  led0 : out std_logic; -- heartbeat led
  hex0_d : out std_logic_vector(6 downto 0); -- seven segement display 0
  hex0_dp : out std_logic; -- decimal point
  hex1_d : out std_logic_vector(6 downto 0); -- seven segement display 1
  hex1_dp : out std_logic; -- decimal point
  hex2_d : out std_logic_vector(6 downto 0); -- seven segement display 2
  hex2_dp : out std_logic; -- decimal point
  hex3_d : out std_logic_vector(6 downto 0); -- seven segement display 3
  hex3_dp : out std_logic -- decimal point
);
end dg_de0;

architecture impl of dg_de0 is

component pll01 IS
	PORT
	(
		areset		: IN STD_LOGIC;
		inclk0		: IN STD_LOGIC;
		c0		: OUT STD_LOGIC ;
		locked		: OUT STD_LOGIC 
	);
END component;


component clkgen is
  port
  (
    clk_sys : in std_logic; -- system clock
    reset : in std_logic; -- reset
    hour_set : in std_logic; -- '1' = hour set mode
    minute_set : in std_logic; -- '1' = minute set mode
    sec_pulse : in std_logic; -- second pulse
    minutes : out std_logic_vector(5 downto 0); -- minutes output
    hours : out std_logic_vector(4 downto 0) -- hours out
  );
end component;

component calc_segdata is
  port
  (
    hours : in std_logic_vector(4 downto 0);
    minutes : in std_logic_vector(5 downto 0);
    ht_seg : out std_logic_vector(7 downto 0);
    ho_seg : out std_logic_vector(7 downto 0);
    mt_seg : out std_logic_vector(7 downto 0);
    mo_seg : out std_logic_vector(7 downto 0)   
  );    
end component;


constant cntr_max : integer := 5_000_000;
-- ceil(log2(cntr_max) = 23
constant cntr_size : integer := 23;

signal clk5 : std_logic;
signal sec_pulse : std_logic;
signal cntr_reg : unsigned(cntr_size-1 downto 0) := (others => '0');
signal cntr_next : unsigned(cntr_size-1 downto 0);

signal reset : std_logic;
signal hour_set : std_logic;
signal minute_set : std_logic;
signal hours_buf : std_logic_vector(4 downto 0);
signal minutes_buf : std_logic_vector(5 downto 0);
 
signal dispdata_buf : std_logic_vector(31 downto 0);
signal dispdata_buf_n : std_logic_vector(31 downto 0);

signal seg_reg : std_logic_vector(7 downto 0);
signal seg_next : std_logic_vector(7 downto 0);
signal digitsel_reg : std_logic_vector(3 downto 0);
signal digitsel_next : std_logic_vector(3 downto 0);
 
begin
	reset <= not btn0_n;
	hour_set <= not btn2_n;
	minute_set <= not btn1_n;
	
  pll_inst : pll01
  port map
  (
    areset => '0',
		inclk0 => clk50,
		c0 => clk5,
		locked => open
  );

  process (clk5)
  begin
    if rising_edge(clk5) then
      cntr_reg <= cntr_next;
      seg_reg <= seg_next;
      digitsel_reg <= digitsel_next;
    end if;
  end process;
  cntr_next <= (others => '0') when (cntr_reg = (cntr_max-1)) else (cntr_reg + 1);
  sec_pulse <= '1' when cntr_reg = (cntr_max-1) else '0';
  led0 <= cntr_reg(cntr_size-1);


  clkg : clkgen
  port map
  (
    clk_sys => clk5,
    reset => reset,
    hour_set => hour_set,
    minute_set => minute_set,
    sec_pulse => sec_pulse,
    minutes => minutes_buf,
    hours => hours_buf
  );
  
  csegdata : calc_segdata
  port map
  (
    hours => hours_buf,
    minutes => minutes_buf,
    ht_seg => dispdata_buf(31 downto 24),
    ho_seg => dispdata_buf(23 downto 16),
    mt_seg => dispdata_buf(15 downto 8),
    mo_seg => dispdata_buf(7 downto 0)
  );
  
  dispdata_buf_n <= not dispdata_buf;
  

  hex0_d <= dispdata_buf_n(6 downto 0);
  hex0_dp <= dispdata_buf_n(7);
  hex1_d <= dispdata_buf_n(14 downto 8);
  hex1_dp <= dispdata_buf_n(15);
  hex2_d <= dispdata_buf_n(22 downto 16);
  hex2_dp <= dispdata_buf_n(23);
  hex3_d <= dispdata_buf_n(30 downto 24);
  hex3_dp <= dispdata_buf_n(31);
  
end impl;
