-- trans_segdata_a.vhd

library ieee;
use ieee.std_logic_1164.all;

entity trans_segdata_a is
  generic
  (
    cntr_size : integer := 20
  );
  port
  (
    clk : in std_logic;
    ht_seg : in std_logic_vector(7 downto 0);
    ho_seg : in std_logic_vector(7 downto 0);
    mt_seg : in std_logic_vector(7 downto 0);
    mo_seg : in std_logic_vector(7 downto 0);
    seg : out std_logic_vector(6 downto 0); -- seven segement display
    dp : out std_logic; -- decimal point
    an : out std_logic_vector(3 downto 0) -- anode drivers (inverted)
  );   
end trans_segdata_a;

architecture impl of trans_segdata_a is

component sseg_control is
  generic
  (
    cntr_size : integer := 20
  );
  port
  (
    clk : in std_logic;
    disp_data : in std_logic_vector(31 downto 0);
    seg_data : out std_logic_vector(7 downto 0);
    digit_sel : out std_logic_vector(3 downto 0)
  );
end component;

signal dispdata_buf : std_logic_vector(31 downto 0);

signal seg_reg : std_logic_vector(7 downto 0);
signal seg_next : std_logic_vector(7 downto 0);
signal digitsel_reg : std_logic_vector(3 downto 0);
signal digitsel_next : std_logic_vector(3 downto 0);
 

begin

  process (clk)
  begin
    if rising_edge(clk) then
      digitsel_reg <= digitsel_next;
      seg_reg <= seg_next;
    end if;
  end process;


  dispdata_buf <= ht_seg & ho_seg & mt_seg & mo_seg;

  disp_control : sseg_control
  generic map
  (
    cntr_size => cntr_size
  )
  port map
  (
    clk => clk,
    disp_data => dispdata_buf,
    seg_data => seg_next,
    digit_sel => digitsel_next
  );
  
  seg <= seg_reg(6 downto 0);
  dp <= seg_reg(7);
  an <= not digitsel_reg;
  
end impl;
