-- calc_segdata.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity calc_segdata is
  port
  (
    hours : in std_logic_vector(4 downto 0);
    minutes : in std_logic_vector(5 downto 0);
    ht_seg : out std_logic_vector(7 downto 0);
    ho_seg : out std_logic_vector(7 downto 0);
    mt_seg : out std_logic_vector(7 downto 0);
    mo_seg : out std_logic_vector(7 downto 0)   
  );    
end calc_segdata;

architecture impl of calc_segdata is


component bin6tobcd is
  port
  (
    bin6 : in std_logic_vector(5 downto 0);
    ones : out std_logic_vector(3 downto 0);
    tens : out std_logic_vector(2 downto 0)
  );
end component;

component hex2sseg is
  port
  (
    hexnum : in std_logic_vector(3 downto 0);
    decpoint : in std_logic;
    sevseg : out std_logic_vector(7 downto 0) -- dp : bit 7; g: bit 6 ; ... a : bit 0
  );
end component;


constant hours_size : integer := 5;
constant minutes_size : integer := 6;


signal hours_tens : std_logic_vector(2 downto 0);
signal hours_ones : std_logic_vector(3 downto 0);
signal minutes_tens : std_logic_vector(2 downto 0);
signal minutes_ones : std_logic_vector(3 downto 0);

signal hours_val : std_logic_vector(5 downto 0);
signal hours_tens_val : std_logic_vector(3 downto 0);
signal minutes_tens_val : std_logic_vector(3 downto 0);

begin

  hours_val <= '0' & hours;
  hours_tens_val <= '0' & hours_tens;
  minutes_tens_val <= '0' & minutes_tens;
  
  hours_bcd : bin6tobcd
  port map
  (
    bin6 => hours_val,
    ones => hours_ones,
    tens => hours_tens
  );
  
  minutes_bcd : bin6tobcd
  port map
  (
    bin6 => minutes,
    ones => minutes_ones,
    tens => minutes_tens
  );

  minutes_ones_sseg : hex2sseg
  port map
  (
    hexnum => minutes_ones,
    decpoint => '0',
    sevseg => mo_seg
  );
  
  minutes_tens_sseg : hex2sseg
  port map
  (
    hexnum => minutes_tens_val,
    decpoint => '0',
    sevseg => mt_seg
  );
  
  hours_ones_sseg : hex2sseg
  port map
  (
    hexnum => hours_ones,
    decpoint => '0',
    sevseg => ho_seg
  );
  
  hours_tens_sseg : hex2sseg
  port map
  (
    hexnum => hours_tens_val,
    decpoint => '0',
    sevseg => ht_seg
  );
  
end impl;
