-- clkgen_tb

library ieee;
use ieee.std_logic_1164.all;

entity clkgen_tb is
end clkgen_tb;

architecture sim of clkgen_tb is
component clkgen is
  port
  (
    clk_sys : in std_logic; -- system clock
    reset : in std_logic; -- reset
    -- hour_set : in std_logic; -- '1' = hour set mode
    -- minute_set : in std_logic; -- '1' = minute set mode
    sec_pulse : in std_logic; -- second pulse
    minutes : out std_logic_vector(5 downto 0); -- minutes output
    hours : out std_logic_vector(4 downto 0) -- hours out
  );
end component;

signal clk_sim : std_logic := '0';
signal reset_sim : std_logic := '0';
signal sec_pulse_sim : std_logic := '0';
signal minutes_sim : std_logic_vector(5 downto 0) := (others => '0');
signal hours_sim: std_logic_vector(4 downto 0) := (others => '0');

constant clkp : time := 0.5ns;

begin
  dut : clkgen
  port map
  (
    clk_sys => clk_sim,
    reset => reset_sim,
    sec_pulse => sec_pulse_sim,
    minutes => minutes_sim,
    hours => hours_sim
  );
  
  -- clock generation
  process
  begin
    clk_sim <= '1';
    wait for clkp/2;
    clk_sim <= '0';
    wait for clkp/2;
  end process;
  
  process (clk_sim)
  begin
    if rising_edge(clk_sim) then
      sec_pulse_sim <= not sec_pulse_sim;
    end if;
  end process;
  
  process
  begin
    reset_sim <= '1';
    wait for clkp*2;
    reset_sim <= '0';
    wait;
  end process;
   
end sim;
