-- clkgen_step2.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity clkgen is
  port
  (
    clk_sys : in std_logic; -- system clock
    reset : in std_logic; -- reset
    -- hour_set : in std_logic; -- '1' = hour set mode
    -- minute_set : in std_logic; -- '1' = minute set mode
    sec_pulse : in std_logic; -- second pulse
    minutes : out std_logic_vector(5 downto 0); -- minutes output
    hours : out std_logic_vector(4 downto 0) -- hours out
  );
end clkgen;

architecture impl of clkgen is

constant sec_per_min : integer := 60; -- seconds per minute
constant min_per_hour : integer := 60; --minutes per hour
constant hours_per_day : integer := 24; -- hours per day

constant seccntr_size : integer := 6; -- ceil(log2(sec_per_minute))
constant mincntr_size : integer := 6; -- ceil(log2(min_per_hour))
constant hourcntr_size : integer := 5; -- ceil(log2(hours_per_day))

-- counter for seconds per miunte (0 ... 59)
signal seccntr_reg : unsigned(seccntr_size-1 downto 0) := (others => '0');
signal seccntr_next : unsigned(seccntr_size-1 downto 0);

-- counter for minutes per hour (0 ... 59)
signal mincntr_reg : unsigned(mincntr_size-1 downto 0) := (others => '0');
signal mincntr_next: unsigned(mincntr_size-1 downto 0);

-- counter for hours per day (0 ... 23)
signal hourcntr_reg : unsigned(hourcntr_size-1 downto 0) := (others => '0');
signal hourcntr_next : unsigned(hourcntr_size-1 downto 0);

signal min_pulse : std_logic;
signal hour_pulse : std_logic;

begin

  process (clk_sys, sec_pulse, reset)
  begin
    if rising_edge(clk_sys) then
      if reset = '1' then
        seccntr_reg <= (others => '0');
        mincntr_reg <= (others => '0');
        hourcntr_reg <= (others => '0');
      elsif sec_pulse = '1' then
        seccntr_reg <= seccntr_next;
        mincntr_reg <= mincntr_next;
        hourcntr_reg <= hourcntr_next;
      end if;
    end if;
  end process;


  seccntr_next <= (others =>'0') when seccntr_reg = (sec_per_min - 1) else seccntr_reg + 1;
  
  min_pulse <= '1' when seccntr_reg  = (sec_per_min - 1) else '0';
  hour_pulse <= '1' when (mincntr_reg  = (min_per_hour - 1)) and  (seccntr_reg  = (sec_per_min - 1)) else '0';
  
  process (min_pulse, mincntr_reg)
  begin
    mincntr_next <= mincntr_reg;  -- default
    if min_pulse = '1' then
      if mincntr_reg = (min_per_hour -1) then
        mincntr_next <= (others => '0');
      else
        mincntr_next <= mincntr_reg + 1;
      end if;
    end if;
  end process;
  
  process (hour_pulse, hourcntr_reg)
  begin
    hourcntr_next <= hourcntr_reg; -- default
    if hour_pulse = '1' then
      if hourcntr_reg = (hours_per_day -1) then
        hourcntr_next <= (others => '0');
      else
        hourcntr_next <= hourcntr_reg + 1;
      end if;
    end if;
  end process;
  
  minutes <= std_logic_vector(mincntr_reg);
  hours <= std_logic_vector(hourcntr_reg);  
end impl;
