-- rgbm_test3.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity rgbm_test3 is
  port
  (
    clk100 : in std_logic;
    btnC : in std_logic;
    r0 : out std_logic;
    r1 : out std_logic;
    g0 : out std_logic;
    g1 : out std_logic;
    b0 : out std_logic;
    b1 : out std_logic;
    a : out std_logic;
    b : out std_logic;
    c : out std_logic;
    d : out std_logic;
    clk_pix : out std_logic;
    stb : out std_logic;
    oe_n : out std_logic
  );
end rgbm_test3;

architecture impl of rgbm_test3 is
signal clk5 : std_logic;
signal reset : std_logic;
signal oe : std_logic;

constant rsize : integer := 6;
signal x_addr : std_logic_vector(rsize-1 downto 0);
signal y_addr : std_logic_vector(rsize-1 downto 0);
signal disp_yaddr : std_logic_vector(3 downto 0);
signal disp_yaddr_minus_one : std_logic_vector(3 downto 0);

signal p_x : unsigned(rsize-1 downto 0);
signal p_y : unsigned(rsize-1 downto 0);

signal upper_obj : std_logic_vector(2 downto 0);
signal lower_obj : std_logic_vector(2 downto 0);

component clk_wiz_0
  port
  (
    clk_in1 : in std_logic;
    clk_out1 : out std_logic
  ); 
end component;

component rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
  );
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic;
    oe_len : in std_logic_vector(rsize-1 downto 0);
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end component;


begin

  clk_mgnt : clk_wiz_0
  port map
  (
    clk_in1 => clk100,
    clk_out1 => clk5
  ); 

  matrix_cntrl : rgbm_cntr
   port map
  (
    clk_sys => clk5,
    reset => reset,
    oe_len => (0|1|2 => '1', others => '0'),
    led_x => x_addr,
    led_y => y_addr,
    pix_clk => open,
    pix_clk_g => clk_pix,
    stb => stb,
    oe => oe
  );
    
  oe_n <= not oe;
  reset <= btnC;
  
  disp_yaddr <= y_addr(3 downto 0);
  disp_yaddr_minus_one <= std_logic_vector(unsigned(disp_yaddr) - 1);
  p_x <= unsigned(x_addr);
  p_y <= unsigned(y_addr);
  
  upper_obj <= "100" when (p_y = 1) and (p_x <= 4)
                      else "001" when ((5 <= p_y) and (p_y <= 10)
                              and (12 <= p_x) and (p_x <= 17))
                      else "000";
  lower_obj <= "010" when (p_y = 2) and (p_x <= 9) else "000";
  
  
  a <= disp_yaddr_minus_one(0);
  b <= disp_yaddr_minus_one(1);
  c <= disp_yaddr_minus_one(2);
  d <= disp_yaddr_minus_one(3);
  
  r0 <= upper_obj(2);
  g0 <= upper_obj(1);
  b0 <= upper_obj(0);
  
  r1 <= lower_obj(2);
  g1 <= lower_obj(1);
  b1 <= lower_obj(0);
  
end impl;
