-- rgbm_test4.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity rgbm_test4 is
  port
  (
    clk100 : in std_logic;
    btnC : in std_logic;
    r0 : out std_logic;
    r1 : out std_logic;
    g0 : out std_logic;
    g1 : out std_logic;
    b0 : out std_logic;
    b1 : out std_logic;
    a : out std_logic;
    b : out std_logic;
    c : out std_logic;
    d : out std_logic;
    clk_pix : out std_logic;
    stb : out std_logic;
    oe_n : out std_logic
  );
end rgbm_test4;

architecture impl of rgbm_test4 is
signal clk5 : std_logic;
signal reset : std_logic;
signal oe : std_logic;

constant rsize : integer := 6;
signal x_addr : std_logic_vector(rsize-1 downto 0);
signal y_addr : std_logic_vector(rsize-1 downto 0);
signal disp_yaddr : std_logic_vector(3 downto 0);
signal disp_yaddr_minus_one : std_logic_vector(3 downto 0);

signal p_x : unsigned(rsize-1 downto 0);
signal p_y : unsigned(rsize-1 downto 0);

signal obj_u : std_logic_vector(2 downto 0);
signal obj_l : std_logic_vector(2 downto 0);

signal rect_active_u : std_logic;
signal rect_active_l : std_logic;
constant rect_color : std_logic_vector(2 downto 0) := "001"; -- blue

component clk_wiz_0
  port
  (
    clk_in1 : in std_logic;
    clk_out1 : out std_logic
  ); 
end component;

component rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
  );
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic;
    oe_len : in std_logic_vector(rsize-1 downto 0);
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end component;


begin

  clk_mgnt : clk_wiz_0
  port map
  (
    clk_in1 => clk100,
    clk_out1 => clk5
  ); 

  matrix_cntrl : rgbm_cntr
   port map
  (
    clk_sys => clk5,
    reset => reset,
    oe_len => (0|1|2 => '1', others => '0'),
    led_x => x_addr,
    led_y => y_addr,
    pix_clk => open,
    pix_clk_g => clk_pix,
    stb => stb,
    oe => oe
  );
    
  oe_n <= not oe;
  reset <= btnC;
  
  disp_yaddr <= y_addr(3 downto 0);
  disp_yaddr_minus_one <= std_logic_vector(unsigned(disp_yaddr) - 1);
  p_x <= unsigned(x_addr);
  p_y <= unsigned(y_addr);
  
  rect_active_u <= '1' when (((12 <= p_x) and (p_x <= 17))
                             and ((13 <= p_y) and (p_y <= 15))) 
                             else '0';
  rect_active_l <= '1' when (((12 <= p_x) and (p_x <= 17))
                             and ((0 <= p_y) and (p_y <= 2))) 
                             else '0';
                             
  proc_u : process(rect_active_u)
  begin
    obj_u <= "000"; -- blank screen
    if rect_active_u = '1' then
      obj_u <= rect_color;
    end if;
  end process;

  proc_l : process(rect_active_l)
  begin
    obj_l <= "000"; -- blank screen
    if rect_active_l = '1' then
      obj_l <= rect_color;
    end if;
  end process;
  
  
  a <= disp_yaddr_minus_one(0);
  b <= disp_yaddr_minus_one(1);
  c <= disp_yaddr_minus_one(2);
  d <= disp_yaddr_minus_one(3);
  
  r0 <= obj_u(2);
  g0 <= obj_u(1);
  b0 <= obj_u(0);
  
  r1 <= obj_l(2);
  g1 <= obj_l(1);
  b1 <= obj_l(0);
  
end impl;
