-- rgbm_test6.vhd

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity rgbm_test6 is
  port
  (
    clk100 : in std_logic;
    btnC : in std_logic;
    r0 : out std_logic;
    r1 : out std_logic;
    g0 : out std_logic;
    g1 : out std_logic;
    b0 : out std_logic;
    b1 : out std_logic;
    a : out std_logic;
    b : out std_logic;
    c : out std_logic;
    d : out std_logic;
    clk_pix : out std_logic;
    stb : out std_logic;
    oe_n : out std_logic
  );
end rgbm_test6;

architecture impl of rgbm_test6 is
signal clk5 : std_logic;
signal reset : std_logic;
signal oe : std_logic;

constant rsize : integer := 6;
signal x_addr : std_logic_vector(rsize-1 downto 0);
signal y_addr : std_logic_vector(rsize-1 downto 0);
signal disp_yaddr : std_logic_vector(3 downto 0);
signal disp_yaddr_minus_one : std_logic_vector(3 downto 0);

signal p_x : unsigned(rsize-1 downto 0);
signal p_y : unsigned(rsize-1 downto 0);
signal p_y_l : unsigned(rsize-1 downto 0);

signal obj_u : std_logic_vector(2 downto 0);
signal obj_l : std_logic_vector(2 downto 0);

signal rect_active_u : std_logic;
signal rect_active_l : std_logic;
constant rect_color : std_logic_vector(2 downto 0) := "001"; -- blue
constant rect_size : integer := 6;
constant rect_vp : unsigned(rsize-1 downto 0) := to_unsigned(1,rsize);
constant rect_vn : unsigned(rsize-1 downto 0) := unsigned(to_signed(-1,rsize));
constant rect_xl : unsigned := to_unsigned(12, rsize);
constant rect_xr : unsigned := rect_xl + rect_size - 1;

signal rect_yt : unsigned(rsize-1 downto 0) := to_unsigned(13, rsize);
signal rect_yb : unsigned(rsize-1 downto 0) := rect_yt + rect_size - 1;
signal rect_yreg : unsigned(rsize-1 downto 0) := (others => '0');
signal rect_ynext : unsigned(rsize-1 downto 0);
signal rect_dyreg : unsigned(rsize-1 downto 0) := rect_vp;
signal rect_dynext : unsigned(rsize-1 downto 0);

signal obj_reg_u : std_logic_vector(2 downto 0) := "000";
signal obj_next_u : std_logic_vector(2 downto 0);

signal obj_reg_l : std_logic_vector(2 downto 0) := "000";
signal obj_next_l : std_logic_vector(2 downto 0);

signal timer_reg : unsigned(17 downto 0) := (others => '0');
signal timer_next : unsigned(17 downto 0);
signal upd_pulse : std_logic;

component clk_wiz_0
  port
  (
    clk_in1 : in std_logic;
    clk_out1 : out std_logic
  ); 
end component;

component rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
  );
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic;
    oe_len : in std_logic_vector(rsize-1 downto 0);
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end component;


begin

  clk_mgnt : clk_wiz_0
  port map
  (
    clk_in1 => clk100,
    clk_out1 => clk5
  ); 

  matrix_cntrl : rgbm_cntr
   port map
  (
    clk_sys => clk5,
    reset => reset,
    oe_len => (0|1|2 => '1', others => '0'),
    led_x => x_addr,
    led_y => y_addr,
    pix_clk => open,
    pix_clk_g => clk_pix,
    stb => stb,
    oe => oe
  );
   
  process (clk5)
  begin
    if rising_edge(clk5) then
      if reset = '1' then
        obj_reg_u <= "000"; -- blank screen
        obj_reg_l <= "000"; -- blank screen
        rect_yreg <= (others => '0');
        rect_dyreg <= rect_vp;
        timer_reg <= (others => '0');
      else
        obj_reg_u <= obj_next_u;
        obj_reg_l <= obj_next_l;
        rect_yreg <= rect_ynext;
        rect_dyreg <= rect_dynext;
        timer_reg <= timer_next;
      end if;
    end if;
  end process;
  
  timer_next <= timer_reg + 1;
  upd_pulse <= '1' when timer_reg = 0 else '0';    
  oe_n <= not oe;
  reset <= btnC;
  
  disp_yaddr <= y_addr(3 downto 0);
  disp_yaddr_minus_one <= std_logic_vector(unsigned(disp_yaddr) - 1);
  p_x <= unsigned(x_addr);
  p_y <= unsigned(y_addr);
  p_y_l <= p_y + 16; 
  
  rect_active_u <= '1' when (((rect_xl <= p_x) and (p_x <= rect_xr))
                             and (((rect_yt <= p_y) and (p_y <= rect_yb))
                              or ((rect_yt <= p_y) and (p_y_l <= rect_yb)))) 
                             else '0';
                             
  rect_active_l <= '1' when (((rect_xl <= p_x) and (p_x <= rect_xr))
                             and (((rect_yt <= p_y_l) and (p_y_l <= rect_yb))
                              or ((rect_yt <= p_y) and (p_y_l <= rect_yb)))) 
                             else '0';


  rect_yt <= rect_yreg;
  rect_yb <= rect_yt + rect_size - 1;
  
  rect_ynext <= rect_yreg + rect_dyreg when upd_pulse = '1' else rect_yreg;

  proc_rect : process(rect_dyreg, rect_yt, rect_yb)
  begin
    -- default value
    rect_dynext <= rect_dyreg;
    
    if rect_yt < 1 then -- if rect reaches top reverse direction
      rect_dynext <= rect_vp;
    elsif rect_yb > (31 - 1) then
      rect_dynext <= rect_vn; -- if rect reaches bottom reverse direction
    end if;
  end process proc_rect;
                            
  proc_u : process(rect_active_u)
  begin
    obj_next_u <= "000"; -- blank screen
    if rect_active_u = '1' then
      obj_next_u <= rect_color;
    end if;
  end process;

  proc_l : process(rect_active_l)
  begin
    obj_next_l <= "000"; -- blank screen
    if rect_active_l = '1' then
      obj_next_l <= rect_color;
    end if;
  end process;
  
  
  a <= disp_yaddr_minus_one(0);
  b <= disp_yaddr_minus_one(1);
  c <= disp_yaddr_minus_one(2);
  d <= disp_yaddr_minus_one(3);
  
  r0 <= obj_reg_u(2);
  g0 <= obj_reg_u(1);
  b0 <= obj_reg_u(0);
  
  r1 <= obj_reg_l(2);
  g1 <= obj_reg_l(1);
  b1 <= obj_reg_l(0);
  
end impl;
