-- rgbm_cntr_tb.vhd

library ieee;
use ieee.std_logic_1164.all;

entity rgbm_cntr_tb is
end rgbm_cntr_tb;

architecture sim of rgbm_cntr_tb is
component rgbm_cntr is
  generic
  (
    row_count : integer := 32;
    rows_per_frame : integer := 2;
    col_count : integer := 32;
    rsize : integer := 6;
    stb_len : integer := 2 -- length of stb pulse in pix_clk cycles
  );
  port
  (
    clk_sys : in std_logic;
    reset : in std_logic := '0';
    oe_len : in std_logic_vector(rsize-1 downto 0) := (0|1|2 => '1', others => '0');
    led_x : out std_logic_vector(rsize-1 downto 0);
    led_y : out std_logic_vector(rsize-1 downto 0);
    pix_clk : out std_logic;
    pix_clk_g : out std_logic;
    stb : out std_logic;
    oe : out std_logic
  );
end component;

constant rsize : integer := 6;
signal clk_sim : std_logic;
signal reset_sim : std_logic;
signal oe_len_sim : std_logic_vector(rsize-1 downto 0);
signal led_x_sim : std_logic_vector(rsize-1 downto 0);
signal led_y_sim : std_logic_vector(rsize-1 downto 0);
signal pix_clk_sim : std_logic;
signal pix_clk_g_sim : std_logic;
signal stb_sim : std_logic;
signal oe_sim : std_logic;

constant clkp : time := 20ns;

begin
  dut : rgbm_cntr
  port map
  (
    clk_sys => clk_sim,
    reset => reset_sim,
    oe_len => oe_len_sim,
    led_x => led_x_sim,
    led_y => led_y_sim,
    pix_clk => pix_clk_sim,
    pix_clk_g => pix_clk_g_sim,
    stb => stb_sim,
    oe => oe_sim
  );
  
  clk_proc : process
  begin
    clk_sim <= '1';
    wait for clkp/2;
    clk_sim <= '0';
    wait for clkp/2;
  end process clk_proc;
  
  sim_proc : process
  begin
    reset_sim <= '1';
    oe_len_sim <= (0|1|2 => '1', others => '0');
    wait for clkp*2;
    reset_sim <= '0';
    wait;
  end process sim_proc;

end sim;