library ieee;
use ieee.std_logic_1164.all;

entity mxo3_fb is
	port
	(
		clk_x1 : in std_logic;
		DIPSW : in std_logic_vector(1 downto 0);
		LED : out std_logic_vector(7 downto 0)
	);
end mxo3_fb;

architecture impl of mxo3_fb is
-- all logic consistently as positive logic
-- switches on board use negative logic and
-- leds are driven on cathodes
signal dipsw_n : std_logic_vector(1 downto 0);
signal led_blink_n : std_logic;
signal led_and_n : std_logic;
signal led_or_n: std_logic;

component fpga_basic is
	port
	(
		clk : in std_logic; -- clock input
    in_1 : in std_logic; -- switch /pushbutton input
    in_2 : in std_logic;  -- switch /pushbutton input
		blink_o : out std_logic; -- output for alternating signal ( ~ 1Hz)
		and_o : out std_logic;  -- output for and operation
    or_o : out std_logic  -- output for or operation
  );
end component;

begin
  -- inversion operations
	dipsw_n <= not DIPSW;
	LED(0) <= not led_blink_n;
	LED(1) <= not led_and_n;
	LED(2) <= not led_or_n;

	fb : fpga_basic
	port map
	(
		clk => clk_x1,
		in_1 => dipsw_n(0),
		in_2 => dipsw_n(1),
		blink_o => led_blink_n,
		and_o => led_and_n,
		or_o => led_or_n
	);
end impl;
